# Description: Constructs intermediate data file from raw data

library(lattice)
#library(latticeExtra)
#library(MASS)

# Clean up workspace and load data
rm(list=ls(all=TRUE))
data<-read.csv('counts.csv',header=T)

# Read chamber volumes
extinctions<-read.csv('extinctions.csv',header=T)
extinctions<-extinctions[extinctions$Chamber.size==700,]
vol<-data.frame(Chamber=extinctions$ID,size=extinctions$Chamber.size,fed=extinctions$No.subchambers.fed); 
data<-merge(data,vol,by='Chamber')

# Format date time class
data$Date2<-NULL
for(i in 1:dim(data)[1]){
	data$Date2[i]<-round(julian(strptime(data$Date[i],'%Y-%m-%d'),origin=as.Date(ifelse(data$ID[i]<=45,'2007-07-30','2007-07-31'))))
}
data$week<-data$Date2/7

# Constructed variables
data$adults<-rowMeans(data[,6:11])
data$gravid<-rowMeans(data[,12:17])
data$juv<-rowMeans(data[,18:23])
data$occupied<-(data$adults+data$juv>0)
data2<-na.omit(data)
data2$Subhab.id<-paste(data2$Chamber,data2$Subhab)

data.food<-read.csv('feeding-levels.csv',header=T)
data.food$Subhab.id<-paste(data.food$Chamber,data.food$Subhab)
tmp<-data.frame(Subhab.id=data.food$Subhab.id,Food=data.food$Food)
data2<-merge(data2,data.frame(Subhab.id=data.food$Subhab.id,Food=data.food$Food, Height=data.food$Height),by='Subhab.id')

#CREATE METAPOPULATION TIME SERIES TABLE
u<-unique(data2$Chamber)
v<-unique(data2$Date2)
data3<-data.frame(NULL)

for(j in 1:length(u)){
	for(k in 1:length(v)){
		tmp<-data2[data2$Chamber==u[j] & data2$Date2==v[k],]
		abundance<-sum(tmp$adults+tmp$juv)

		no.occupied<-sum(tmp$occupied)
		frac.occupied<-sum(tmp$occupied)/dim(tmp)[1]
		adults<-sum(tmp$adults)
		gravid<-sum(tmp$gravid)
		juv<-sum(tmp$juv)

		tmp2<-data.frame(chamber=u[j],day=v[k],week=tmp$week[1], subpops=tmp$Habitats[1], height=tmp$Height[1], size=tmp$size[1],
			fed=tmp$fed[1], abundance=round(abundance), no.occupied=no.occupied, frac.occupied=frac.occupied,
			adults=round(adults), gravid=round(gravid), juv=round(juv))
		data3<-rbind(data3,tmp2)		
	}
}

data3<-na.omit(data3)
write.csv(data3,'metapopulation_series.csv')

# CREATE SUBPOPULATION TABLE
data.subpop<-data.frame(NULL)
u<-unique(data2$Subhab.id)

# RE-ORDER DATA AND COMPUTE SUMMARIES
for(j in 1:length(u)){
	tmp<-data2[data2$Subhab.id==u[j],]		#get all data for one chamber
	tmp<-tmp[order(tmp$week),]			#data get out of order, so sort

	#week level summaries
	tmp$abundance<-(tmp$adults+tmp$juv)

	#series level summaries
	mean.abundance<-mean(tmp$abundance)
	var.abundance<-var(tmp$abundance)
	time.occupied<-sum(tmp$abundance!=0)
	metapop.ext.week<-tmp$week[dim(tmp)[1]]
	mean.juv<-mean(tmp$juv)
	var.juv<-var(tmp$juv)
	mean.adults<-mean(tmp$adults)
	var.adults<-var(tmp$adults)
	fraction.occupied<-(time.occupied)/dim(tmp)[1]
		
	total.changes<-0
	if (dim(tmp)[1]==1) {
		total.changes<-1
		mean.extinction<-1
		var.extinction<-NA
		mean.recolonization<-NA
		var.recolonization<-NA
	} else {
		data.tmp<-data.frame(NULL)
		i0=1
		for(i in 2:dim(tmp)[1]){
			if (tmp$occupied[i]!=tmp$occupied[i-1]){
				total.changes<-total.changes+1	
				T<-tmp$week[i]-tmp$week[i0]
				ifelse(tmp$occupied[i]==TRUE,event<-'recolonization',event<-'extinction')	
				i0=i	
				data.tmp<-rbind(data.tmp,data.frame(T=T,event=event))
			}
		}
	
		if(total.changes==0){
			mean.extinction<-NA
			var.extinction<-NA
			mean.recolonization<-NA
			var.recolonization<-NA
		} else {
			mean.extinction<-mean(data.tmp$T[data.tmp$event=='extinction'])
			var.extinction<-var(data.tmp$T[data.tmp$event=='extinction'])
			ifelse(total.changes>=2,mean.recolonization<-mean(data.tmp$T[data.tmp$event=='recolonization']),
				mean.recolonization<-NA)
			ifelse(total.changes>=2,var.recolonization<-var(data.tmp$T[data.tmp$event=='recolonization']),
				var.recolonization<-NA)
		}
	}
	censored<-tmp$occupied[length(tmp$occupied)]
	source<-tmp$Food[1]>0
	food<-tmp$Food[1]
	no.extinctions<-ceiling(total.changes/2)
	no.recolonizations<-total.changes-no.extinctions

	# Construct record-keeping variables
	data.subpop<-rbind(data.subpop,data.frame(chamber=u[j],subhab=tmp$Subhab.id[1], subpops=tmp$Habitats[1], height=tmp$Height[1], size=tmp$size[1],
		fed=tmp$fed[1], fraction.occupied=fraction.occupied,total.changes=total.changes,censored=censored,
		source=source, food=food, mean.abundance=mean.abundance, var.abundance=var.abundance,no.extinction=no.extinctions,no.recolonizations=no.recolonizations,
		mean.extinction=mean.extinction, var.extinction=var.extinction,
		mean.recolonization=mean.recolonization, var.recolonization=var.recolonization,
		metapop.extinction=metapop.ext.week, observations=dim(tmp)[1],
		mean.juv=mean.juv,var.juv=var.juv,
		mean.adults=mean.adults, var.adults=var.adults))
}

# Some calculated variables
data.subpop$turnover<-data.subpop$total.changes/data.subpop$observations
data.subpop$subpop.size<-data.subpop$size/data.subpop$subpops
data.subpop$food2<-(data.subpop$food>0)

# Save output
write.csv(data.subpop,'data.subpop.csv')

data4<-data2[data2$occupied==TRUE,]
data4$total.size<-data4$adults+data4$juv

# Table data5 will have one record for each Chamber for each day
data5<-aggregate(data4$total.size,by=list(data4$Chamber,data4$Date),sum)
names(data5)<-c('Chamber','Date','total.size')
tmp<-aggregate(data4$adults,by=list(data4$Chamber,data4$Date),sum)[3]; data5$adults<-unlist(tmp)
tmp<-aggregate(data4$juv,by=list(data4$Chamber,data4$Date),sum)[3]; data5$juv<-unlist(tmp)

data.metapop<-data.frame(mean.adults=tapply(data5$adults,data5$Chamber,mean),
	var.adults=tapply(data5$adults,data5$Chamber,var),
	mean.juv=tapply(data5$juv,data5$Chamber,mean),
	var.juv=tapply(data5$juv,data5$Chamber,var),
	mean.total=tapply(data5$total.size,data5$Chamber,mean),
	var.total=tapply(data5$total.size,data5$Chamber,var))

data.metapop$cv.adults<-sqrt(data.metapop$var.adults)/data.metapop$mean.adults
data.metapop$cv.juv<-sqrt(data.metapop$var.juv)/data.metapop$mean.juv
data.metapop$cv.total<-sqrt(data.metapop$var.total)/data.metapop$mean.total

# Save output
write.csv(data.metapop,'data.metapop.csv')
summary.data <- read.csv('data.metapop.csv')
data <- merge(extinctions, summary.data, by.x='ID', by.y='X')
write.csv(data,'data.csv')

# Calculate population size and variability
rm(list=ls(all=TRUE))
data <- read.csv('metapopulation_series.csv')
data <- data[(data$size==700 & data$abundance>0),]
mean <- aggregate(data$abundance, by=list(data$chamber), FUN=mean)
s <- aggregate(data$abundance, by=list(data$chamber), FUN=sd)
cv <- s[,2]/mean[,2]
data <- data.frame(chamber=mean$Group.1, mean.abundance=mean$x, sd.abundance=s$x, cv=cv)
write.csv(data, file='summary.csv')